from dolfin import *
import sys
set_log_level(PROGRESS)

r = 1
tau = 0.0
gamma = 10.0*r*r
example = int(sys.argv[1])

if (example==0):
    uexact = Expression("(x[0]-pow(x[0],2))*exp(3*x[0])*sin(2*pi*x[1])")
    f = Expression("-exp(3*x[0])*(-9*pow(x[0],2)-3*x[0]+4)*sin(2*pi*x[1]) \
				    +(x[0]-pow(x[0],2))*exp(3*x[0])*4*pow(pi,2)*sin(2*pi*x[1])")
    g = Constant(0.0)
else:
    uexact = Expression("sin(3*pi*x[0])*cos(3*pi*x[1])")
    f = Expression("18*pow(pi,2)*sin(3*pi*x[0])*cos(3*pi*x[1])")
    g = Expression("sin(3*pi*x[0])*cos(3*pi*x[1])")

Th = UnitSquareMesh(33,33)
Vh = FunctionSpace(Th, "DG", r)

u = TrialFunction(Vh)
v = TestFunction(Vh)
n = FacetNormal(Th)
h = CellSize(Th)

a = dot(grad(u), grad(v))*dx \
  - dot(jump(v, n), avg(grad(u)))*dS \
  - tau*dot(jump(u, n), avg(grad(v)))*dS \
  + gamma/avg(h)*dot(jump(u, n), jump(v, n))*dS \
  - v*dot(grad(u), n)*ds \
  - tau*u*dot(grad(v), n)*ds \
  + gamma/h*u*v*ds
L = f*v*dx - tau*g*dot(grad(v), n)*ds + gamma/h*g*v*ds

u = Function(Vh)
solve(a==L, u)

print "L2 error: ", errornorm(uexact, u)
plot(u, interactive=True)
