from dolfin import *
import sys

# ******************************************************************************
def poisson(deg,x1,x2,y1,y2,m,f):

	Th = IntervalMesh(m,x1,x2)
	Vh = FunctionSpace(Th,"CG",deg)

	# define boundary conditions
	def uBoundary1(x):
		return abs(x[0]-x1)<1e-12
	def uBoundary2(x):
		return abs(x[0]-x2)<1e-12
	bc = [DirichletBC(Vh, y1, uBoundary1), DirichletBC(Vh, y2, uBoundary2)]

	# define functions
	u = TrialFunction(Vh)
	v = TestFunction(Vh)

	# define the problem
	a = inner(grad(u), grad(v))*dx
	L = f*v*dx

	# assemble stifness matrix and load vector (with bc)
	A, b = assemble_system(a, L, bc)

	# solve the problem
	u = Function(Vh)
	solve(A, u.vector(), b)

	return u
	
# ******************************************************************************

# get degree
degree = int(sys.argv[1])

# boundary points
x1 = -2*pi
x2 = 2*pi

# Dirichlet B.C.
y1 = 1.0
y2 = 1.0

# right-hand side of the equation
ue = Expression("cos(x[0])")
f = Expression("cos(x[0])")

# errors
L2error = []
H1error = []

m = 20
for i in range(0, 5):
    u = poisson(degree,x1,x2,y1,y2,m,f)
    L2 = errornorm(ue, u, norm_type="L2")
    H10 = errornorm(ue, u, norm_type="H10")
    H1error.append(L2+H10)
    L2error.append(L2)
    m *= 2

print "degree = ", degree

from math import log as ln
print "\nExpected order", degree
for i in range(0, 4):
    print "H1 order:", ln(H1error[i]/H1error[i+1])/ln(2.)
print "\nExpected order", degree + 1
for i in range(0, 4):
    print "L2 order:", ln(L2error[i]/L2error[i+1])/ln(2.)

# plot
plot(u)
interactive()

